import { Component, OnInit } from '@angular/core';
import { Observable } from 'rxjs';
import { Store, select } from '@ngrx/store';
import * as utils from '@app/degree-planner/shared/utils';

import { SavedForLaterCourse } from '@app/core/models/saved-for-later-course';
import { Course } from '@app/core/models/course';

// rsjx / ngrx
import { DegreePlannerState } from '@app/degree-planner/store/state';
import {
  AddSaveForLater,
  RemoveCourse,
} from '@app/degree-planner/store/actions/course.actions';
import * as selectors from '@app/degree-planner/store/selectors';
import { distinctUntilChanged } from 'rxjs/operators';
import { TermCode } from '@app/core/models/termcode';

@Component({
  selector: 'cse-saved-for-later-container',
  templateUrl: './saved-for-later-container.component.html',
  styleUrls: ['./saved-for-later-container.component.scss'],
})
export class SavedForLaterContainerComponent implements OnInit {
  public courses$: Observable<SavedForLaterCourse[]>;
  public dropZoneIds$: Observable<string[]>;

  constructor(private store: Store<{ degreePlanner: DegreePlannerState }>) {}

  public ngOnInit() {
    this.dropZoneIds$ = this.store.pipe(
      select(selectors.selectAllVisibleYears),
      utils.yearsToDropZoneIds(),
      distinctUntilChanged(utils.compareStringArrays),
    );

    this.courses$ = this.store.pipe(select(selectors.getSavedForLaterCourses));
  }

  drop(event: any) {
    const newContainerId = event.container.id;
    const previousContainerId = event.previousContainer.id;
    const fromTerm = previousContainerId.indexOf('term-') === 0;
    const fromDifferentContainer = newContainerId !== previousContainerId;

    if (fromDifferentContainer && fromTerm) {
      const course = event.item.data as Course;

      this.store.dispatch(
        new AddSaveForLater({
          courseId: course.courseId,
          subjectCode: course.subjectCode,
          title: course.title,
          catalogNumber: course.catalogNumber,
          newIndex: event.currentIndex,
        }),
      );

      this.store.dispatch(
        new RemoveCourse({
          fromTermCode: new TermCode(course.termCode),
          recordId: course.id as number,
        }),
      );
    }
  }
}
